/*
  AVE TS Websocket multiclient
  
  Test:
  atr = new DominaplusWSInterface();
  atr.addServer("UNO", "10.30.92.7");
  atr.addServer("DUE", "10.30.93.251");
  //atr.setListener(function(ev, srv, data){ console.log(srv); console.log(data); console.log("-----"); });
  atr.connectServers();

  atr.sendWSCommandByName("UNO", "EBI", "1,10")
  atr.sendWSCommandByName("DUE", "EBI", "1,10")


*/
////////////////////////////////////////////////////////////////////////////////
// Class
function DominaplusWSInterface(){
  
  this.serverIPs = [];
	this.serverNames = [];
	this.serverDeviceTypes = [];
  this.servers = [];
  this.port = "14001";
  this.watchDogInterval = undefined;
  this.watchDogTimeout = 20000;
	this.doLog = true;
	this.buildingData = [];
  
  //////////////////////////////////////////////////////////////////////////////
  // Class constructor 
  if (!(this instanceof DominaplusWSInterface)) {
	  return new AveTSRemote(ip)
  };

  //////////////////////////////////////////////////////////////////////////////
  // Default event listener
  this.listener = function(ev, server, data){
    var me = this;
    if(this.doLog) console.log("Event: [" + ev + "] [" + server.srvname + " - " + server.ipaddress + "]: Command " + data.command );
  };
  //////////////////////////////////////////////////////////////////////////////
  // Default interface listener
  this.interfaceListener = function(ipSupervisor, deviceCode, deviceTypeCode){
  
  };
  //////////////////////////////////////////////////////////////////////////////
  // Keep connections opened
  this.checkConnections = function(){
      
    if(this.watchDogInterval != undefined) {
      clearInterval(this.watchDogInterval);
    }
    
    var me = this;  
    this.watchDogInterval = setInterval(function(){ 
      if(me.doLog) console.log("Checking " + me.servers.length + " connections...")
      for(var cnt = 0; cnt < me.servers.length; cnt++) {
        if(me.doLog) console.log("Check connection status (server: " + me.servers[cnt].aveName + ", interval " + me.watchDogTimeout + ")");
        
        if (me.servers[cnt].readyState === me.servers[cnt].OPEN) {
          server = new Object();
          server.srvname = me.servers[cnt].aveName;
					server.ipaddress = me.servers[cnt].aveIp; 
					server.deviceType = me.servers[cnt].aveDeviceType; 
					server.ev = "watchdog";
          me.listener("watchdog", server, false);
        } else {
          me.servers[cnt] = me.createWsConnection(me.servers[cnt].aveName, me.servers[cnt].aveIp, me.servers[cnt].aveDeviceType, me.servers[cnt].avePort);
        }
      }    
    }, this.watchDogTimeout);
  }
  
  //////////////////////////////////////////////////////////////////////////////
  // Create WS Connection
  this.createWsConnection = function(name, ip, deviceType, port){ 
	
		if(ip == "127.0.0.1") {
			return false;
		}
		
    ws = new WebSocket("ws://" + ip + ":" + port, ['binary','base64']);	
    
    ws.binaryType = 'arraybuffer';
    ws.onmessage = this.messageArrived;	
    
    ws.aveName = name;		
		ws.aveIp = ip;	
		ws.aveDeviceType = deviceType;	
    ws.avePort = port;		
    ws.evListener = this.listener;
    
    var me = this;
    
    ws.onopen = function(ev){
      if(me.doLog) console.log("Websocket connection OPENED [" + ev.target.aveName + "]: " + ev.target.aveIp);
      server = new Object();
      server.srvname = ev.target.aveName;
			server.ipaddress = ev.target.aveIp; 
			server.deviceType = ev.target.aveDeviceType;
			server.ev = "opened";
      me.listener("opened", server, false);
    };
    
    ws.onclose = function(ev){
      if(me.doLog) console.log("Websocket connection CLOSED [" + ev.target.aveName + "]: " + ev.target.aveIp);
      server = new Object();
      server.srvname = ev.target.aveName;
			server.ipaddress = ev.target.aveIp; 
			server.deviceType = ev.target.aveDeviceType;
			server.ev = "closed";
      me.listener("closed", server, false);
    };
    
    return ws;
  }

  //////////////////////////////////////////////////////////////////////////////
  // Open connections 
  this.connectServers = function(){
  
    this.servers = new Array();
    if(this.doLog) console.log("Connecting servers...");
    for(var cnt = 0; cnt < this.serverNames.length; cnt++) {
      if(this.doLog) console.log("Connecting [" + this.serverNames[cnt] + "]");
      this.servers.push(this.createWsConnection(this.serverNames[cnt], this.serverIPs[cnt], this.serverDeviceTypes[cnt], this.port));
    }
    
    if(this.doLog) console.log("Connection done!");
    this.checkConnections();
    if(this.doLog) console.log("Watchdogs activated!");
  };
  
  //////////////////////////////////////////////////////////////////////////////
  // Add a new server IP
  this.setListener = function(listener){
    if(typeof listener == "function") {
      if(this.doLog) console.log("Changing listener...");
      this.listener = listener;
    }
  }
  
  //////////////////////////////////////////////////////////////////////////////
  // Add a new server IP
  this.addServer = function(name, ip, deviceType){
		
    if(name == "") return false;
    if(ip == "") return false;
    if(this.serverNames.indexOf(name) != -1) return false;

    this.serverIPs.push(ip);
    this.serverNames.push(name);
		this.serverDeviceTypes.push(deviceType);
		
    if(this.doLog) console.log("Server added [" + name + "]: " + ip);
    return true;

  };
  
  //////////////////////////////////////////////////////////////////////////////
  // Send a command to a websocket server by name
	this.sendWSCommandByName = function (name, cmd, parameters, records) {
	  for(var i = 0; i < this.serverNames.length; i++) {
	    if(this.serverNames[i] == name) {
	      if(this.doLog) console.log("Sending command [" + cmd + "] to " + this.serverNames[i]);
	      this._sendWSCommand(this.servers[i], cmd, parameters, records);
	    }
	  }
	}
	
  //////////////////////////////////////////////////////////////////////////////
  // Send a command to a websocket server by IP address
	this.sendWSCommandByIP = function (ip, cmd, parameters, records) {
	  for(var i = 0; i < this.serverIPs.length; i++) {
	    if(this.serverIPs[i] == ip) {
	      if(this.doLog) console.log("Sending command [" + cmd + "] to " + this.serverIPs[i]);
	      this._sendWSCommand(this.servers[i], cmd, parameters, records);
	    }
	  }
	}
	
  //////////////////////////////////////////////////////////////////////////////
  // Send a command to a websocket server
	this._sendWSCommand = function (ws, cmd, parameters, records)
	{
		var str = String.fromCharCode(0x02)+cmd;
		var message = "";
		if (parameters)
		{
			message+=String.fromCharCode(0x1d);
			var pieces = parameters.toString().split(",");
			//we need to add one byte (0x1D) for separator for each parameter sent by the user
			var numpar = pieces.length;
			
			for(var i=0; i<numpar;i++)
			{
				message+=pieces[i];
				if (i<(numpar-1))	message+=String.fromCharCode(0x1d);
			}				
		}

		if (records)
		{
			if (!Array.isArray(records)) records=(""+records).split(","); "Vericare perchè importata da TS01"
			for (var record=0; record<records.length; record++)
			{
				message+=String.fromCharCode(0x1e);
				var pieces = records[record].toString().split(",");
				//we need to add one byte (0x1D) for separator for each parameter sent by the user
				var numpar = pieces.length;
				
				for(var i=0; i<numpar;i++)
				{
					message+=pieces[i];
					if (i<(numpar-1))	message+=String.fromCharCode(0x1d);
				}
			}			
		}
		
		str+=message;
		str+=String.fromCharCode(0x03)
		
		if(this.doLog) console.log("Sending command: " + str);
		var CRC = this.buildCRC(str);			
		
		if (ws.readyState === ws.OPEN)
		{			
			ws.send(str+CRC+String.fromCharCode(0x04));
		}
		else
		{ 
		  if(this.doLog) console.log("ERROR: Websocket is not connected!");
		}
	};
	
  //////////////////////////////////////////////////////////////////////////////
  // Message received event
	this.messageArrived = function (response)
	{
	  if(this.doLog) console.log("Message arrived, processing...");
	  
		var bytearray = new Uint8Array(response.data);
		
		var globalstr = "";
		if(bytearray[1] == 102 || bytearray[2] == 97 || bytearray[3] == 108 || bytearray[4] == 112)
		{
			console.log('FALP Special case');
			for (var i=0;i<bytearray.length;i++)
			{
				globalstr +=  String.fromCharCode(bytearray[i]);
			}
		} else {
			for (var i=0;i<bytearray.length;i++)
			{
				globalstr += '%' + ('0' + bytearray[i].toString(16)).slice(-2);
			}
			globalstr = decodeURIComponent(globalstr);
		}

		var messaggi = globalstr.split(String.fromCharCode(0x04));

		for (var m=0;m<messaggi.length;m++)
		{			
				
			var str = messaggi[m].substr(1,messaggi[m].length-4);
			var pieces = str.split(String.fromCharCode(0x1e)); 
			
			var records0=pieces[0];	
			var campirecord0 = records0.split(String.fromCharCode(0x1d));
			var command=campirecord0[0];
			var parameters = new Array();
			var records = new Array();
			
			if (campirecord0.length>1)
			{
				for (var i=1;i<campirecord0.length;i++)
				{
					parameters.push(campirecord0[i]);					
				}
			}
			
			
			if (pieces.length > 1) 
			{ 
				var nrecords=pieces.length-1;
				for (var i=1;i<pieces.length;i++)
				{
					var currentRecord = pieces[i];
					var campiRecord = currentRecord.split(String.fromCharCode(0x1d));
					records.push(campiRecord);
					var nCampiCurrentRecord = campiRecord.length;
				}
			}	

		  server = new Object();
		  server.srvname = response.target.aveName;
		  server.ipaddress = response.target.aveIp; 
			server.deviceType = response.target.aveDeviceType; 
			server.ev = "msg";
			
		  data = new Object();
		  data.command = command;
		  data.parameters = parameters;
		  data.records = records;
		
		  if(typeof response.target.evListener == "function") {
		    response.target.evListener("msg", server, data);
      }
			
		}
	};
	
  
  //////////////////////////////////////////////////////////////////////////////
  // Value to HEX 
  this.valueToHex = function(value)
	{
		var valueHex = 0;
		if (value>9)
		{
			switch (value)
			{
				case 10: valueHex = "A"; break;
				case 11: valueHex = "B"; break;
				case 12: valueHex = "C"; break;
				case 13: valueHex = "D"; break;
				case 14: valueHex = "E"; break;
				case 15: valueHex = "F"; break;
			}
		}
		else
		{
			valueHex = ""+value;
		}
		return valueHex;
	};


  //////////////////////////////////////////////////////////////////////////////
  // Build the CRC
	this.buildCRC = function(globalString)
	{
		var crc = 0;
		for (var i=0;i<globalString.length;i++)
		{
			crc = crc ^ globalString.charCodeAt(i);
		}
		crc = 0xFF - crc;
		var MSB = this.valueToHex(parseInt(crc/16));
		var LSB = this.valueToHex(parseInt(crc%16));
		crc = MSB+LSB;
		return crc;	
	};
	
	
}
