// valida il codice fiscale
jQuery.validator.addMethod("codiceFiscale", function(value, element) {
	return this.optional(element) || validaCodiceFiscale(element);
}, $.validator.format("Codice fiscale non corretto"));

//valida la partita iva
jQuery.validator.addMethod("partitaIva", function(value, element) {
	return this.optional(element) || validaPartitaIva(element);
}, $.validator.format("Partita IVA non corretta"));

//controlla che il campo sia o un codice fiscale o una partita iva
jQuery.validator.addMethod("partitaIvaOrCodiceFiscale", function(value, element) {
	return this.optional(element) || (validaPartitaIva(element) || validaCodiceFiscale(element)); 
}, $.validator.format("Inserire o un codice fiscale o una partita IVA"));

//controlla che il campo sia lungo esattamente tot caratteri
jQuery.validator.addMethod("exactlength", function(value, element, params) {
    return this.optional(element) || value.length == params;
}, $.validator.format("Inserisci esattamente {0} caratteri."));

//controlla che una data sia in formato aaaa-mm-gg hh:mm:ss
jQuery.validator.addMethod("dateTimeISO", function(value, element) {
	return this.optional(element) || /^(\d{2}|\d{4})(?:\-)?([0]{1}\d{1}|[1]{1}[0-2]{1})(?:\-)?([0-2]{1}\d{1}|[3]{1}[0-1]{1})(?:\s)?([0-1]{1}\d{1}|[2]{1}[0-3]{1})(?::)?([0-5]{1}\d{1})(?::)?([0-5]{1}\d{1})?$/.test(value);
}, $.validator.format("Please enter a correct date"));

//controlla che una data orario (senza secondi) sia in formato gg-mm-aaaa hh:mm
jQuery.validator.addMethod("dateTimeITA", function(value, element) {
	return this.optional(element) || /^(0?[1-9]|[12][0-9]|3[01])[\/\-\.](0?[1-9]|1[012])[\/\-\.](\d{4})[\ ](0?[0-9]|1[0-9]|2[0-4])[\:]([0-5])([0-9])$/.test(value);
}, $.validator.format("Inserisci una data e un orario corretti"));

//compilazione condizionata
jQuery.validator.addMethod("requiredIF", function(value, element, params) {
	return controllaCompilazioneCondizionata(value, element, params);
}, $.validator.format("Il campo è obbligatorio."));

//compilazione condizionata
jQuery.validator.addMethod("notCompilableIF", function(value, element, params) {
	return controllaNonCompilareCampoSe(value, element, params);;
}, $.validator.format("Dati incongruenti: il campo non può essere compilato."));

// numero decimale
jQuery.validator.addMethod("decimale", function(value, element) {
	//return this.optional(element) || /^-?(?:\d+|\d{1,3}(?:\.\d{3})+)(?:,\d+)?$/.test(value);
	return this.optional(element) || /^[0-9,]*$/.test(value);
}, $.validator.format("Inserisci un numero decimale nel formato 00000,00"));

//compilazione condizionata
jQuery.validator.addMethod("checkOreTotali", function(value, element, params) {
	return checkOreTotali(value, element, params);
}, $.validator.format("Attenzione: le ore inserite non sono coerenti con la pianificazione."));

//dimensione massima del file da caricare
jQuery.validator.addMethod("maxSizeFile", function(value, element) {
	if(element.files.length > 0 && element.files[0].size > MAX_SIZE_FILE_UPLOAD){
		return false;
	}
	else{
		return true;
	}
}, $.validator.format(JQUERY_VALIDATE_ERRORE_MAX_FILE_SIZE_UPLOAD.replace('##max_file_size_upload##', (MAX_SIZE_FILE_UPLOAD/1024/1024))));

//almeno uno all'interno di un gruppo
jQuery.validator.addMethod( "require_from_group", function( value, element, options ) {
	var $fields = $( options[ 1 ], element.form ),
		$fieldsFirst = $fields.eq( 0 ),
		validator = $fieldsFirst.data( "valid_req_grp" ) ? $fieldsFirst.data( "valid_req_grp" ) : $.extend( {}, this ),
		isValid = $fields.filter( function() {
			return validator.elementValue( this );
		} ).length >= options[ 0 ];

	// Store the cloned validator for future validation
	$fieldsFirst.data( "valid_req_grp", validator );

	// If element isn't being validated, run each require_from_group field's validation rules
	if ( !$( element ).data( "being_validated" ) ) {
		$fields.data( "being_validated", true );
		$fields.each( function() {
			validator.element( this );
		} );
		$fields.data( "being_validated", false );
	}
	return isValid;
}, $.validator.format( "Compilare almeno {0} di questi campi." ) );

//estensioni file
$.validator.addMethod( "extension", function( value, element, param ) {
	param = typeof param === "string" ? param.replace( /,/g, "|" ) : "png|jpe?g|gif";
	return this.optional( element ) || value.match( new RegExp( "\\.(" + param + ")$", "i" ) );
}, $.validator.format( "Tipo di documento non valido" ) );

/*********************************************************************************************************/
//FUNZIONE CHE CONTROLLA LA COMPILAZIONE CONDIZIONATA
/*********************************************************************************************************/
function controllaCompilazioneCondizionata(value, element, params) {
	if (params.indexOf('!=') != -1) {
		var a = params.split('!=');
		var valoreCampoCheCondiziona = $('#'+a[0]).val();
		if ( a[1].indexOf('|') != -1 ) {
			b = a[1].split('|');
		} else {
			b = new Array();
			b[0] = a[1];
		}
		if (!in_array(valoreCampoCheCondiziona, b)) {
			if (Boolean(value)) {
				return true;
			} else {
				return false;
			}
		} else {
			return true;
		}
		return false;
	} else if (params.indexOf('=') != -1 ) {
		var a = params.split('=');
		var valoreCampoCheCondiziona = $('#'+a[0]).val();
		if ( a[1].indexOf('|') != -1 ) {
			b = a[1].split('|');
		} else {
			b = new Array();
			b[0] = a[1];
		}
		if (in_array(valoreCampoCheCondiziona, b)) {
			if (Boolean(value)) {
				return true;
			} else {
				return false;
			}
		} else {
			return true;
		}
		return false;
	} else {
		if ($('#'+params).attr('type')=='checkbox') {
			if ( Boolean( $('#'+params).attr('checked') ) ) {
				return (value != '');
			} else {
				return true;
			}
		} else {
			if ( Boolean( $('#'+params).val() ) ) {
				return (value != '');
			} else {
				return true;
			}
		}
	}
}
/*********************************************************************************************************/
//FUNZIONE CHE CONTROLLA LA COMPILAZIONE CONDIZIONATA
/*********************************************************************************************************/
function controllaNonCompilareCampoSe(value, element, params) {
	if (params.indexOf('=') != -1 ) {
		var a = params.split('=');
		//alert('a='+a);
		var valoreCampoCheCondiziona = $('input[name="'+a[0]+'"]:radio:checked').val();
		//alert('valoreCampoCheCondiziona='+valoreCampoCheCondiziona);
		if ( a[1].indexOf('|') != -1 ) {
			b = a[1].split('|');
		} else {
			b = new Array();
			b[0] = a[1];
		}
		//alert('b='+b);
		if (in_array(valoreCampoCheCondiziona, b)) {
			if ($(element).is('input[type="checkbox"]') || $(element).is('input[type="radio"]')) {
				return !$(element).is('_checked');
			} else {
				return !Boolean(value);
			}
		}
	}
	/*
	return true;
	
	if (params.indexOf('!=') != -1 ) {
		var a = params.split('!=');
		var valoreCampoCheCondiziona = $('#'+a[0]).val();
		if ( a[1].indexOf('|') != -1 ) {
			b = a[1].split('|');
		} else {
			b = new Array();
			b[0] = a[1];
		}
		if (!in_array(valoreCampoCheCondiziona, b)) {
			if (Boolean(value)) {
				//alert('1');
				return false;
			} else {
				//alert('2');
				return true;
			}
		} else {
			//alert('3');
			return false;
		}
		//alert('4');
		return true;
	} else if (params.indexOf('=') != -1 ) {
		var a = params.split('=');
		//alert('a='+a);
		var valoreCampoCheCondiziona = $('#'+a[0]).val();
		if ( a[1].indexOf('|') != -1 ) {
			b = a[1].split('|');
		} else {
			b = new Array();
			b[0] = a[1];
		}
		//alert('valoreCampoCheCondiziona='+valoreCampoCheCondiziona+':');
		//alert('b='+b+':');
		if (in_array(valoreCampoCheCondiziona, b)) {
			if (Boolean(value)) {
				//alert('5');
				return false;
			} else {
				//alert('6');
				return true;
			}
		} else {
			//alert('7');
			return false;
		}
		//alert('8');
		return true;
	} else {
		if ( Boolean( $('#'+params).attr('checked') ) ) {
			//alert('9');
			return (value == '');
		} else {
			//alert('10');
			return false;
		}
	}
	*/
}
/*********************************************************************************************************/
//FUNZIONE CHE CONTROLLA LA COMPILAZIONE CONDIZIONATA
/*********************************************************************************************************/
function checkOreTotali(value, element, params) {
	var tot = timepickerMinuteDiff('lunedi') + 
	          timepickerMinuteDiff('martedi') + 
	          timepickerMinuteDiff('mercoledi') +
    		  timepickerMinuteDiff('giovedi') + 
              timepickerMinuteDiff('venerdi') + 
              timepickerMinuteDiff('sabato') + 
              timepickerMinuteDiff('domenica');
	value = parseFloat(value.replace(',','.'));
	return (value*60 == tot);
}
/*********************************************************************************************************/
function timepickerMinuteDiff(giorno) {
	var date1 = new Date(2000, 1, 1, $('#'+giorno+'_orario_da').val().substring(0,2), $('#'+giorno+'_orario_da').val().substring(3,5),0);
	var date2 = new Date(2000, 1, 1, $('#'+giorno+'_orario_a').val().substring(0,2), $('#'+giorno+'_orario_a').val().substring(3,5),0);
	return Math.floor((date2 - date1)/1000/60);
}
/*********************************************************************************************************/
//FUNZIONE CHE CONTROLLA LA VALIDITA' DEL CODICE FISCALE
/*********************************************************************************************************/
function validaCodiceFiscale(obj) {
	var nTempNum;
	var nAppoNum;
	var i;
	var szTempCodFisc;
	var szLastChar;
	var szValued;
	var szInValued;
	 
	szCodFisc = obj.value;
	 
	if (szCodFisc.length < 16) {
	  return false;
	} else if (szCodFisc == "") {
	  return false;
	} else {
		szCodFisc =szCodFisc.toUpperCase();
		szLastChar = szCodFisc.substr(szCodFisc.length - 1,1);
		szTempCodFisc =szCodFisc.substr(0,szCodFisc.length - 1);
		nTempNum = 0;
		i = 0;
		
		while (true) {
			szValued="B1A0KKPPLLC2QQD3RRE4VVOOSSF5TTG6UUH7MMI8NNJ9WWZZYYXX";
			// I DISPARI
			szInValued=szTempCodFisc.substr(i,1);
			nAppoNum = szValued.search(szInValued);
			nTempNum = nTempNum + (nAppoNum &  0x7FFE) / 2;
			i = i + 1;
			if (i>15) {
				break;
			}
			szValued="A0B1C2D3E4F5G6H7I8J9KKLLMMNNOOPPQQRRSSTTUUVVWWXXYYZZ";
			// I PARI
			szInValued=szTempCodFisc.substr(i,1);
			nAppoNum = szValued.search(szInValued);
			nTempNum =  nTempNum + (nAppoNum &  0x7FFE) / 2;
			i = i + 1;
		}
		nTempNum = nTempNum % 26; //mod
		szValued="ABCDEFGHIJKLMNOPQRSTUVWXYZ";
		szValued= szValued.substr(nTempNum, 1);
		if (szValued==szLastChar) {
			return true;
		} else {
			return false;
		}
	}
}
/*********************************************************************************************************/
//FUNZIONE CHE CONTROLLA LA PARTITA IVA
/*********************************************************************************************************/
function validaPartitaIva(objPi) {
	pi = objPi.value;
	if( pi == '' ) {
		return false;
	}
	if( pi.length != 11 ) {
		return false;
	}
	validi = "0123456789";
	for( i = 0; i < 11; i++ ){
		if( validi.indexOf( pi.charAt(i) ) == -1 ) {
			return false;
		}
	}
	s = 0;
	for( i = 0; i <= 9; i += 2 ) {
		s += pi.charCodeAt(i) - '0'.charCodeAt(0);
	}
	for( i = 1; i <= 9; i += 2 ) {
		c = 2*( pi.charCodeAt(i) - '0'.charCodeAt(0) );
		if( c > 9 )  c = c - 9;
		s += c;
	}
	if( ( 10 - s%10 )%10 != pi.charCodeAt(10) - '0'.charCodeAt(0) ) {
		return false;
	}
	return true;
}
/*********************************************************************************************************/
//FUNZIONE CHE CONTROLLA LA PARTITA IVA
/*********************************************************************************************************/
function in_array (needle, haystack, argStrict) {
  // http://kevin.vanzonneveld.net
  // +   original by: Kevin van Zonneveld (http://kevin.vanzonneveld.net)
  // +   improved by: vlado houba
  // +   input by: Billy
  // +   bugfixed by: Brett Zamir (http://brett-zamir.me)
  // *     example 1: in_array('van', ['Kevin', 'van', 'Zonneveld']);
  // *     returns 1: true
  // *     example 2: in_array('vlado', {0: 'Kevin', vlado: 'van', 1: 'Zonneveld'});
  // *     returns 2: false
  // *     example 3: in_array(1, ['1', '2', '3']);
  // *     returns 3: true
  // *     example 3: in_array(1, ['1', '2', '3'], false);
  // *     returns 3: true
  // *     example 4: in_array(1, ['1', '2', '3'], true);
  // *     returns 4: false
  var key = '',
    strict = !! argStrict;

  if (strict) {
    for (key in haystack) {
      if (haystack[key] === needle) {
        return true;
      }
    }
  } else {
    for (key in haystack) {
      if (haystack[key] == needle) {
        return true;
      }
    }
  }

  return false;
}

