#!/bin/bash

VERSION="1"

ME=$0
CMD=$1
MY_NAME="$(basename $0)"

YASS_OPTIONS=" --sleep 1000 --log 1 "
RUNDIR="/usr/local/ave_rsa"
PKGDIR="/usr/local/ave_rsa_pkg"

FIRE_ALARM_IP=`/bin/cat /usr/local/ave_rsa_pkg/data/fire_alarm_supervisor_ip.txt`
FIRE_ALARM_LAUNCHER="/usr/local/bin/mono /usr/local/ave_rsa/fire_alarm/FireAlarmTCP.exe --server_port 7010 --fa_device_ip $FIRE_ALARM_IP --command_log_level 2 --device_log_level 1"

# FIRE_ALARM_LAUNCHER="/usr/local/bin/mono /usr/local/ave_rsa/fire_alarm/FireAlarmTCP.exe --server_port 7010 --fa_device_ip 10.30.17.10 --command_log_level 2 --device_log_level 1"


YASS_CLIENT_BIN="$RUNDIR/client/yass3client"
YASS_WEB_ROOT="$RUNDIR/web/"
YASS_SQL_ROOT="$RUNDIR/sql/"

SOFTWAREPKG=`ls -a $PKGDIR/yass3*.bin | tail -n 1`
    
LEGACYDATA_DIR="/usr/local/ave_hotel_json"
LEGACYDEFAULTDATA_DIR="/usr/local/ave_rsa/web/hotel/json"

WWWFINAL="/var/www/ave_rsa/web"

date > "$ME.last_exec.txt"

if [ "$(id -u)" != "0" ]; then
   echo "NOT_ROOT"
   exit 1
fi

if [ "$#" -ne 1 ]; then
   echo "PARAM_MISSING"
   exit 1
fi

printHelp() 
{
    echo "-----------------------------------------------------------------"
    echo ":: YASS LAUNCHER v.$VERSION USAGE ::"
    echo " "

    echo "#> ./$MY_NAME help (prints this help)"
    echo "#> ./$MY_NAME ver (prints launcher version)"
    echo "#> ./$MY_NAME check_and_run (check if launcher is running and run it if not)"
    echo "#> ./$MY_NAME check_launcher (check launcher status)"
    echo "#> ./$MY_NAME check_yass (check service status)"
    echo "#> ./$MY_NAME stop_launcher (stop launcher)"
    echo "#> ./$MY_NAME stop_yass (stop yass service, stop launcher before if you want to avoid auto-restart)"
    echo "#> ./$MY_NAME stop_all (stop both launcher and yass service)"
    echo "#> ./$MY_NAME run (run yass in manual [blocking] mode)"
    echo "-----------------------------------------------------------------"
}

dbExists() 
{
    mysqlshow -u root -paveroot2u | grep -v "grep" | grep "ave_rsa" > /dev/null
    if [ $?  -eq 0 ]; then
        return 1
    fi
    return 0
}

launcherRunning() 
{
    ps fax | grep -v "grep" | grep "$ME run" > /dev/null
    if [ $?  -eq 0  ]; then
        return 1
    fi
    return 0
}

mkTmpFs() 
{
    if cat /proc/mounts | grep "$RUNDIR" &> /dev/null; then
        echo "YASS folder already mounted"
    else
        echo "YASS folder not mounted, mounting now..."

        mount -t tmpfs -o size=30m tmpfs $RUNDIR

        if cat /proc/mounts | grep "$RUNDIR" &> /dev/null; then
            echo "Loading software..."
            
            tar -xf $SOFTWAREPKG --warning=no-timestamp -C /

            chown root:root $YASS_WEB_ROOT/runasroot
            chmod +s $YASS_WEB_ROOT/runasroot

            /bin/bash $RUNDIR/post-extract.sh
            /bin/bash $RUNDIR/gpio-btn-reset.sh > /dev/null 2>&1 &

            linkLegacyData

            crontab < /usr/local/ave_rsa/ave_cron
            echo "Loading done!"
        else
            echo "Error mounting tmpfs, loading aborted"
            exit 1
        fi
    fi
}

linkLegacyData()
{
    if [ -d "$LEGACYDATA_DIR" ]; then
        echo "Directory $LEGACYDATA_DIR already exists, nothing to do"
    else
        echo "Directory $LEGACYDATA_DIR NOT exists, creating"
        mkdir -p $LEGACYDATA_DIR
        cp -Rf $LEGACYDEFAULTDATA_DIR/* $LEGACYDATA_DIR
        
    fi

    rm -Rf $LEGACYDEFAULTDATA_DIR
    ln -s $LEGACYDATA_DIR $LEGACYDEFAULTDATA_DIR
    chmod -R 777 $LEGACYDEFAULTDATA_DIR
}

setSystem()
{


    # if [ -d "$WWWFINAL" ]; then
    #     echo "Directory $WWWFINAL already exists, deleting it"
    #     rm -f $WWWFINAL
    # fi

    dbExists
    if [ $? = 1 ]; then
        echo "Database already exists"
    else
        mysql -u root -paveroot2u < $YASS_SQL_ROOT/full_v1.sql
        echo "Creating base DB"
    fi

    ldconfig
    apache2ctl restart

    rm $WWWFINAL
    ln -s $YASS_WEB_ROOT $WWWFINAL
}

launcherRun()
{
    $ME run 1>/dev/null 2>&1 & 
}

launcherStop()
{
    killall $MY_NAME
}

faRunning() 
{
    ps fax | grep -v "grep" | grep "$FIRE_ALARM_LAUNCHER" > /dev/null
    if [ $?  -eq 0 ]; then
        return 1
    fi
    return 0
}

yassRunning() 
{
    ps fax | grep -v "grep" | grep "$YASS_CLIENT_BIN $YASS_OPTIONS" > /dev/null
    if [ $?  -eq 0 ]; then
        return 1
    fi
    return 0
}

yassStop()
{
    screen -S yass -X quit
}

yassLaunch() 
{
    yassStop
    screen -dmS yass bash -c "$YASS_CLIENT_BIN $YASS_OPTIONS ;sleep 1; quit"
}

faStop()
{
    screen -S fa -X quit
}

monoExists()
{    
    if [ -x "$(command -v /usr/local/bin/mono)" ]; then
        echo "MONO_EXISTS"
    else
        echo "MONO_NOT_EXISTS"
    fi    
}

faLaunch() 
{
    if [ -x "$(command -v /usr/local/bin/mono)" ]; then
        faStop
        screen -dmS fa bash -c "$FIRE_ALARM_LAUNCHER ;sleep 1; quit"
    fi
}


if [ $CMD = "help" ]; then
    printHelp
fi

if [ $CMD = "ver" ]; then
    echo $VERSION
fi

if [ $CMD = "check_and_run" ]; then
    launcherRunning
    if [ $? = 1 ]; then
        echo "LAUNCHER_RUNNING"
    else
        echo "LAUNCHER_STARTED"
        launcherRun
    fi
fi

if [ $CMD = "check_launcher" ]; then
    launcherRunning
    if [ $? = 1 ]; then
        echo "LAUNCHER_RUNNING"
    else
        echo "LAUNCHER_NOT_RUNNING"
    fi
fi

if [ $CMD = "stop_all" ]; then
    echo "STOPPING_ALL"
    yassStop
    faStop
    launcherStop
fi

if [ $CMD = "stop_launcher" ]; then
    launcherRunning
    if [ $? = 1 ]; then
        echo "LAUNCHER_KILLING"
        launcherStop
    else
        echo "LAUNCHER_NOT_RUNNING"
    fi
fi

if [ $CMD = "check_yass" ]; then
    yassRunning
    if [ $? = 1 ]; then
        echo "YASS_RUNNING"
    else
        echo "YASS_STOPPED"
    fi
fi

if [ $CMD = "stop_yass" ]; then
    yassRunning
    if [ $? = 1 ]; then
        echo "YASS_STOPPING"
        yassStop
    else
        echo "YASS_ALREADY_STOPPED"
    fi
fi

if [ $CMD = "check_fa" ]; then
    faRunning
    if [ $? = 1 ]; then
        echo "FA_RUNNING"
    else
        echo "FA_STOPPED"
    fi
fi

if [ $CMD = "stop_fa" ]; then
    faRunning
    if [ $? = 1 ]; then
        echo "FA_STOPPING"
        faStop
    else
        echo "FA_ALREADY_STOPPED"
    fi
fi

if [ $CMD = "run" ]; then
    
    hwclock -s -f /dev/rtc1

    sleep 1
    mkTmpFs
    setSystem
    
    if cat /proc/mounts | grep "$RUNDIR" &> /dev/null; then
        while true
        do
            yassRunning
            if [ $? = 1 ]; then
                echo "YASS_ALREADY_RUNNING"
            else
                echo "YASS_LAUNCHING"
                yassLaunch
            fi

            faRunning
            if [ $? = 1 ]; then
                echo "FA_ALREADY_RUNNING"
            else
                echo "FA_LAUNCHING"
                faLaunch
            fi

            sleep 2;
        done
    fi
fi
